﻿using C1.DataCollection;
using C1.DataCollection.Serialization;
using System.Net.Http.Json;
using System.Text.Json;
using FlexGridIntro.Shared;
using System.Threading;

namespace FlexGridIntro.Client.Models
{
    public class VirtualDataCollection<T> : C1CursorDataCollection<T> where T : class
    {
        private string _q;

        private readonly HttpClient Http;
        private readonly string URL;
        private int TotalCount;
        public int PageSize { get; set; } = 15;
        //public event EventHandler<bool> DataFetching;
        private bool _isMoreItems { get; set; } = true;
        public VirtualDataCollection(HttpClient http, string apiURL) 
        {
            this.Http = http;
            this.URL = apiURL;
            
            
        }
        public override bool HasMoreItems
        {
            get { return _q != null && _isMoreItems; }

        }

        //private SemaphoreSlim _searchSemaphore = new SemaphoreSlim(1);

        public async Task SearchAsync(string q)
        {
            //Sets the filter string and wait the Delay time.
            _q = q;
            try
            {
                //await _searchSemaphore.WaitAsync();

                if (_q != q)//If the text changed means there was another keystroke.
                    return;

                await RefreshAsync();
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
            }
            finally
            {
                //_searchSemaphore.Release();
            }
        }
        
        public override bool CanSort(params SortDescription[] sortDescriptions)
        {
            return true;
        }

        public override bool CanRemove(int index)
        {
            return true;   
        }
        public override bool CanFilter(FilterExpression? filterExpression)
        {
            return !(filterExpression is FilterPredicateExpression);
        }

        
        
        //protected override async Task<Tuple<int, IReadOnlyList<T>>> GetPageAsync(int pageIndex, int startingIndex, int count, IReadOnlyList<SortDescription>? sortDescriptions = null, FilterExpression? filterExpression = null, CancellationToken cancellationToken = default)
        //{
        //    string url = $"{URL}?skip={startingIndex}&take={count}";
        //    //DataFetching.Invoke(this, true);
        //    if(this.TotalCount == 0)
        //    {
        //        // Get TotalCount to virtualize Data; This method should be define in the API
        //        this.TotalCount = await Http.GetFromJsonAsync<int>(new Uri($"{URL}/totalcount", UriKind.Relative), cancellationToken);
        //    }
        //    if (sortDescriptions?.Count > 0)
        //    {
        //        url += $"&sort={Uri.EscapeDataString(JsonSerializer.Serialize<IReadOnlyList<SortDescription>>(sortDescriptions))}";
        //    }
        //    var grpup = this.GetGroupDescriptions<T>();
        //    if (filterExpression != null)
        //    {
        //        var options = new JsonSerializerOptions { Converters = { new FilterExpressionJsonConverter() } };
        //        url += $"&filter={Uri.EscapeDataString(JsonSerializer.Serialize(filterExpression, options))}";
        //    }
        //    var response = await Http.GetFromJsonAsync<IEnumerable<T>>(new Uri(url, UriKind.Relative), cancellationToken);
        //    Console.WriteLine($"From API Class {startingIndex} to {startingIndex+count}");
        //    //DataFetching.Invoke(this, false);
        //    return new Tuple<int, IReadOnlyList<T>>(TotalCount,response.ToList());

        //}

        protected override async Task<Tuple<string?, IReadOnlyList<T>>> GetPageAsync(int startingIndex, string? pageToken, int? count = null, IReadOnlyList<SortDescription>? sortDescriptions = null, FilterExpression? filterExpression = null, CancellationToken cancellationToken = default)
        {
            count = PageSize;
            string url = $"{URL}?skip={startingIndex}&take={count}";

            filterExpression= new FilterTextExpression("Customer", FilterOperation.Contains, _q, false, false);
            //DataFetching.Invoke(this, true);
            if (this.TotalCount == 0)
            {
                // Get TotalCount to virtualize Data; This method should be define in the API
                this.TotalCount = await Http.GetFromJsonAsync<int>(new Uri($"{URL}/totalcount", UriKind.Relative), cancellationToken);
            }
            if (sortDescriptions?.Count > 0)
            {
                url += $"&sort={Uri.EscapeDataString(JsonSerializer.Serialize<IReadOnlyList<SortDescription>>(sortDescriptions))}";
            }
            var grpup = this.GetGroupDescriptions<T>();
            if (filterExpression != null)
            {
                var options = new JsonSerializerOptions { Converters = { new FilterExpressionJsonConverter() } };
                url += $"&filter={Uri.EscapeDataString(JsonSerializer.Serialize(filterExpression, options))}";
            }
            var response = await Http.GetFromJsonAsync<Tuple<bool,IEnumerable<T>>>(new Uri(url, UriKind.Relative), cancellationToken);
            
            Console.WriteLine($"From API Class {startingIndex} to {startingIndex + count} & {response.Item1}");
            //DataFetching.Invoke(this, false);
            _isMoreItems = response.Item1;
           
            return new Tuple<string, IReadOnlyList<T>>(pageToken, response.Item2.ToList());
            
        }
    }
}
